from can import Message
from cantools import database
import pandas as pd
from cantools.database import DecodeError
import os
# 载入DBC文件
dbc_path = 'ByNav_X2_CAN_v0.1.dbc'
db = database.load_file(dbc_path)

# 解析ASC文件
asc_path = '4.asc'

# 初始化解析结果存储列表
messages = []

index = 0
# 读取并解析ASC文件
with open(asc_path, 'r') as file:
    for line in file:
        index = index + 1
        parts = line.strip().split(' ')
        if index < 4 or len(parts) < 4:
            continue  # 过滤掉非数据行
        timestamp = float(parts[0])
        channel = parts[1]
        if parts[2].endswith('x'):
            # 如果parts[2]的最后一个字符是'x'，则移除这个字符
            arb_id_str = parts[2][:-1]
        else:
            arb_id_str = parts[2]
        arb_id = int(arb_id_str, 16)  # 将十六进制的仲裁ID转换为整数
        direction = parts[3]  # 'Rx' 或 'Tx', 根据实际情况可能需要或不需要这个字段
        dlc = int(parts[5], 16)  # 数据长度，这里假设数据长度字段是十六进制的，根据你的数据实际情况可能需要调整
        data = bytes(int(byte, 16) for byte in parts[6:6+dlc])
        # 尝试解析消息
        try:
            message = db.decode_message(arb_id, data)
            message['timestamp'] = timestamp
            messages.append(message)
        except DecodeError as e:
            #print(f"DecodeError for message ID {arb_id}: {e}")
            continue
        except KeyError:
            #print(f"Warning: Message with ID {arb_id} not found in DBC.")
            continue
        except ValueError:
            # 如果某条消息在DBC文件中未定义，这里会捕获到异常
            continue

# 转换为DataFrame以便于查看
df_messages = pd.DataFrame(messages)

df_messages.head()

# 初始化列表来存储纬度和经度的数据组合
unique_data = []

# 初始化变量
last_timestamp = 0
latitude = None
longitude = None
heading = None

last_latitude = 0
last_longitude = 0
# 遍历DataFrame的每一行
for index, row in df_messages.iterrows():
    # 提取当前行的时间戳、纬度和经度
    timestamp = row.get('timestamp')
    
    if not pd.isna(row.get('INS_latitude')):
        latitude = round(row.get('INS_latitude'),7)
    if not pd.isna(row.get('INS_longitude')):
        longitude = round(row.get('INS_longitude'),7)
    if not pd.isna(row.get('INS_yawAngle')):
        heading = round(row.get('INS_yawAngle'),7)

    # 检查时间戳间隔是否大于0.02，如果是，则处理纬度和经度值
    if last_timestamp is not None and timestamp - last_timestamp > 0.02:
        # 更新最后的时间戳
        last_timestamp = timestamp
        # 检查纬度和经度是否不为None
        if latitude is not None and longitude is not None:
            formatted_location = f"{timestamp},{latitude},{longitude},{heading}"
            #formatted_location = f"{{{latitude}, {longitude}}},"
            # 检查当前数据组合是否与列表中最后一组不同
            if abs(last_latitude - latitude) > 0.000002 or abs(last_longitude - longitude) > 0.000002:
                # 将新的格式化数据添加到列表中
                unique_data.append(formatted_location)
                last_latitude = latitude
                last_longitude = longitude
    
    

# 将唯一的数据组合转换为DataFrame
df_unique_data = pd.DataFrame(unique_data, columns=['Location'])

# 将DataFrame保存到CSV文件
# 分割文件名和扩展名
base, extension = os.path.splitext(asc_path)
# 替换扩展名
csv_path = base + '.csv'
df_unique_data.to_csv(csv_path, index=False)
